@@ -1,13 +1,13 @@
 ;;; citar-denote.el --- Minor mode integrating Citar and Denote -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2025 Peter Prevos
+;; Copyright (C) 2022-2026 Peter Prevos
 
 ;; Author: Peter Prevos <peter@prevos.net>
 ;; Maintainer: Peter Prevos <peter@prevos.net>
 ;; Homepage: https://github.com/pprevos/citar-denote
-;; Package-Version: 20251220.2230
-;; Package-Revision: 0c04d022ff2c
-;; Package-Requires: ((emacs "28.1") (citar "1.4") (denote "4.1") (dash "2.19.1"))
+;; Package-Version: 20260302.507
+;; Package-Revision: 7b2414c116ff
+;; Package-Requires: ((emacs "29.1") (citar "1.4") (denote "4.1") (dash "2.19.1"))
 
 ;; This file is NOT part of GNU Emacs.
 
@@ -261,24 +261,22 @@ citation key CITEKEY."
 
 (defun citar-denote--get-notes (&optional citekeys)
   "Generate hash table of Denote files associated with CITEKEYS.
-
-If CITEKEYS is omitted, return all Denote files tagged with
-`citar-denote-keyword'."
+If CITEKEYS is omitted, return all Denote files tagged with `citar-denote-keyword'."
   (let ((files (denote-directory-files (concat "_" citar-denote-keyword)))
-	(notes (make-hash-table :test 'equal)))
-    (prog1 notes
-      (dolist (file files)
-        (let ((keys-in-file (citar-denote--retrieve-references file)))
-          (dolist (key keys-in-file)
-            (if citekeys
-                (dolist (k citekeys)
-                  (when (string= k key)
-                    (push file (gethash key notes))))
-              (push file (gethash key notes))))))
-      (maphash
-       (lambda (key filelist)
-         (puthash key (nreverse filelist) notes))
-       notes))))
+        (notes (make-hash-table :test 'equal)))
+    (dolist (file files)
+      (let ((keys-in-file (citar-denote--retrieve-references file)))
+        (dolist (key keys-in-file)
+          (if citekeys
+              (dolist (k citekeys)
+                (when (string= k key)
+                  (push file (gethash key notes))))
+            (push file (gethash key notes))))))
+    (maphash
+     (lambda (key filelist)
+       (puthash key (nreverse filelist) notes))
+     notes)
+    notes))
 
 (defun citar-denote--retrieve-cite-files (citekey)
   "Return names of Denote files that contain CITEKEY.
@@ -505,8 +503,8 @@ There are four special citation macros:
    (cond ((eq citar-denote-signature 'sequence)
           (if (locate-library "denote-sequence")
               (let ((parent (denote-retrieve-filename-signature
-                            (denote-sequence-file-prompt
-                             "New child of SEQUENCE"))))
+                             (denote-sequence-file-prompt
+                              "New child of SEQUENCE"))))
                 (denote-sequence-get-new 'child parent))
             (user-error "Please modify `citar-denote-signature' of install Denote-Sequence")))
          ((eq citar-denote-signature 'ask)
@@ -523,6 +521,21 @@ There are four special citation macros:
     (other-window 1)
     (citar-open-files citekey)))
 
+(defun citar-denote--extract-local-bibliographies ()
+  "Extract local bibliography files from Denote Org files."
+  (let ((files (denote-directory-files (concat "_" citar-denote-keyword ".*\\.org$")))
+	(bib-list '()))
+    (dolist (file files)
+      (with-temp-buffer
+        (insert-file-contents file)
+	(delay-mode-hooks (org-mode))
+        (org-element-map (org-element-parse-buffer) 'keyword
+          (lambda (k)
+            (when (string= (org-element-property :key k) "BIBLIOGRAPHY")
+              (let ((value (org-element-property :value k)))
+                (setq bib-list (append bib-list (split-string value "[ \t]*;[ \t]*" t)))))))))
+    (delete-dups bib-list)))
+
 ;; Interactive functions
 
 ;;;###autoload
@@ -561,22 +574,19 @@ bibliographic entries cited in Denote files."
 ;;;###autoload
 (defun citar-denote-dwim ()
   "Access attachments, notes and links of a bibliographic reference.
-
 When more than one bibliographic item is referenced, select item first."
   (interactive)
-  ;; Any citation keys in the note?
-  (if-let* ((buffer-name (buffer-file-name))
-            (keys (citar-denote--retrieve-references buffer-name))
-            (key (if (= (length keys) 1)
-                     (car keys)
-                   (citar-select-ref
-                    :filter (citar-denote--has-citekeys keys)))))
-      (citar-open (list key))
-    (if (and file (denote-file-is-note-p (buffer-file-name)))
-        (when (yes-or-no-p
-               "Current note does contain a reference. Add one? ")
-          (citar-denote-add-citekey)
-          (citar-denote-dwim))
+  (let ((file (buffer-file-name)))
+    (if (and file (denote-file-is-note-p file))
+        (if-let* ((keys (citar-denote--retrieve-references file))
+                  (key (if (= (length keys) 1)
+                           (car keys)
+                         (citar-select-ref
+                          :filter (citar-denote--has-citekeys keys)))))
+            (citar-open (list key))
+          (when (yes-or-no-p "Current note does not contain a reference. Add one? ")
+            (citar-denote-add-reference)
+            (citar-denote-dwim)))
       (message "Buffer is not a Denote file"))))
 
 ;;;###autoload
@@ -710,9 +720,16 @@ When more than one bibliographic item is referenced, select item first."
 
 ;;;###autoload
 (defun citar-denote-nobib ()
-  "List citation keys referenced or cited in Denote, but not in bibliography."
+  "List citation keys referenced or cited in Denote, but not in the bibliography.
+
+Searches for dead citation keys in the global bibliograhy and local
+bibliographies for Org files with the `citar-denote-keyword'."
   (interactive)
-  (let* ((bibliography (hash-table-keys (citar-get-entries)))
+  (let* ((local-bibs (citar-denote--extract-local-bibliographies))
+         (global-bibs (directory-files ews-bibtex-directory t
+				       "^[A-Z|a-z|0-9].+.bib$"))
+	 (citar-bibliography (append global-bibs local-bibs))
+	 (bibliography (hash-table-keys (citar-get-entries)))
          (citations (citar-denote--extract-citations))
          (references (hash-table-keys (citar-denote--get-notes)))
 	 (union (cl-union citations references))
